/***************************************************************************
*   Copyright (C) 2003 by Hideki Ikemoto                                  *
*   ikemo@users.sourceforge.jp                                            *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
***************************************************************************/

#include "thread.h"

#include <config.h>

#include <stdlib.h>
#include <unistd.h>

#include <kurl.h>
#include <kdebug.h>
#include <kprotocolmanager.h>
#include <kglobal.h>
#include <kstandarddirs.h>

#include <kio/slaveconfig.h>
#include <kio/jobclasses.h>
#include <kio/scheduler.h>
#include <kio/netaccess.h>

#include <qwidget.h>
#include <qapplication.h>
#include <qregexp.h>

#include "board.h"
#include "kita_misc.h"
#include "qcp932codec.h"
#include "threadinfo.h"

using namespace Kita;

QDict<Thread>* Thread::m_threadDict = 0;

Thread::Thread()
{}

Thread::Thread( const QString& datURL, const QString& threadName )
        : m_datURL( datURL ), m_threadName( threadName )
{}

Thread::~Thread()
{}

const QString& Thread::datURL() const
{
    return m_datURL;
}

Thread* Thread::getByURL( const QString& datURL )
{
    if ( m_threadDict == 0 ) {
        m_threadDict = new QDict<Thread>();
    }

    Thread* thread = m_threadDict->find( datURL );
    if ( thread ) return thread;

    Thread* newThread = new Thread( datURL );
    m_threadDict->insert( datURL, newThread );

    return newThread;
}

/* static & public */
Thread* Thread::getByURLNew( const KURL& datURL )
{
    if ( m_threadDict == NULL ) return NULL;
    if ( datURL.isEmpty() ) return NULL;
    return m_threadDict->find( datURL.prettyURL() );
}

const QString& Thread::getName( const KURL& datURL )
{
    Thread* thread = Thread::getByURLNew( datURL );
    if ( thread == NULL ) {
        return QString::null;
    }
    return thread->m_threadName;
}

void Thread::setName( const QString& datURL, QString threadName )
{
    QRegExp qrx(" +$");    
    threadName.replace( qrx, "" );
    
    if ( datURL.isNull() ) {
        return ;
    }

    if ( m_threadDict == 0 ) {
        m_threadDict = new QDict<Thread>();
    }

    if ( m_threadDict->find( datURL ) ) {
        Thread * thread = getByURL( datURL );
        thread->m_threadName = threadName;
    } else {
        Thread* newThread = new Thread( datURL, threadName );
        m_threadDict->insert( datURL, newThread );
    }

    return ;
}

void Thread::replace( const QString& fromURL, const QString& toURL )
{
    QDictIterator<Kita::Thread> it( *m_threadDict );
    for( ; it.current(); ++it ) {
        QString url = it.currentKey();
        Kita::Thread* thread = it.current();
        if ( url.find( fromURL ) == 0 ) {
            m_threadDict->remove( url );
            url = url.replace( 0, fromURL.length(), toURL );
            thread->m_datURL = url;
            Kita::Board::replace( fromURL, toURL );
            m_threadDict->insert( url, thread );
            it.toFirst();
        }
    }
}
